@@ -1,4 +1,4 @@
-r"""This module is to give user a quickly built object detection application
+r"""This module is to give user a quickly built object detection application.
 
 Detection:
     This class provide the object detection function which use Google CLoud Vision API
@@ -7,7 +7,7 @@ Detection:
 Interface:
     This class provide how a GUI for user to save images and see the detected
     images. It can connect to Google Cloud Storage for saving the images being detected.
-    To set the options, please use cfg_template.yaml as template.
+    To set the options, please use cfg_template.yml as template.
 """
 
 # Import built-in library
@@ -45,6 +45,19 @@ LOGGER.setLevel(logging.INFO)
 
 
 class Detection:
+    """Using the Cloud Vision API to get the detected images.
+
+    Attributes:
+        self.img: Original image get from camera.
+        self.frame: Image with bounding boxes on it.
+        self.size: Size of the image and frame.
+        self.categories: Count the detected objects.
+
+    Method:
+        self.start(self): Start the detection.
+        self.end(self): End the detection.
+    """
+
     def __init__(self,
                  google_kit_json_path: str,
                  categories: list,
@@ -113,33 +126,45 @@ class Detection:
         self.__predict_start = False
 
         # Initializing threads
-        detect_objs = threading.Thread(target=self._detect_objs,
+        detect_objs = threading.Thread(target=self.__detect_objs,
                                        daemon=True)
         detect_objs.start()
-        draw_rec = threading.Thread(target=self._draw_rec,
+        draw_rec = threading.Thread(target=self.__draw_rec,
                                     daemon=True)
         draw_rec.start()
-        read_img = threading.Thread(target=self._get_img,
+        read_img = threading.Thread(target=self.__get_img,
                                     daemon=True)
         read_img.start()
 
     @property
     def categories(self):
+        """dict (name, count): Getter of detected categories."""
         return self.__show_categories
 
     @property
     def frame(self):
+        """numpy.array: Getter of detected frame."""
         return self.__frame
 
     @property
     def img(self):
+        """numpy.array: Getter of detected image."""
         return self.__img
 
     @property
     def size(self):
+        """tuple (width, height): Size of the image and frame."""
         return self.__size
 
-    def _detect_objs(self):
+    def __detect_objs(self):
+        """Get the detected information from Cloud Vision API
+        and save the info in self.__detection_info.
+
+        Args:
+            self: Instance itself.
+
+        Return: None
+        """
         while True:
             # Wait for input images
             if (not self.__predict_start) or \
@@ -161,7 +186,14 @@ class Detection:
                                                             ).localized_object_annotations
             cv2.waitKey(30)
 
-    def _get_img(self):
+    def __get_img(self):
+        """Return the image from the camera.
+
+        Args:
+            self: Instance itself.
+
+        Return: None
+        """
         # Read camera image
         while True:
             # Wait for prediction
@@ -176,7 +208,15 @@ class Detection:
 
             self.__img = cv2.resize(self.__img, (self.__size[0], self.__size[1]))
 
-    def _draw_rec(self):
+    def __draw_rec(self):
+        """Draw the bounding boxes and information
+        on the self.__img.
+
+        Args:
+            self: Instance itself.
+
+        Return: None
+        """
         # Draw the bounding boxes to the frame
         while True:
             # Save
@@ -222,6 +262,14 @@ class Detection:
             self.__show_categories = temp_categories
 
     def start(self):
+        """Start the detection.
+
+        Args:
+            self: Instance itself.
+
+        Return: None
+        """
+
         # Wake up the camera and set the size
         self.__cap = cv2.VideoCapture(self.__camera)
         self.__cap.set(3, self.__size[0])
@@ -229,10 +277,18 @@ class Detection:
 
         # Check whether the camera is opened or not
         if not self.__cap.isOpened():
-            LOGGER.error("The camera is not opened, please check the path or the device.")
+            LOGGER.error("The camera is not opened, please check the input or the device.")
         self.__predict_start = True
 
     def end(self):
+        """End the detection and release the resources.
+
+        Args:
+            self: Instance itself.
+
+        Return: None
+        """
+
         # Close the prediction and
         #  release the camera
         self.__predict_start = False
@@ -240,24 +296,35 @@ class Detection:
 
 
 class Interface:
+    """Design the Interface by tkinter and show information on it.
+
+    Attributes:
+        self.frame: Image with bounding boxes on it.
+        self.root: Instance of tkinter.Tk().
+        self.size: Size of the image.
+
+    Method:
+        self.start(self): Start the detection interface.
+    """
+
     def __init__(self,
-                 cfg: str = "cfg.yaml"):
+                 cfg: str = "cfg.yml"):
         # Parse yaml file
         with open(cfg, "r") as yaml_file:
-            self.yaml = yaml.load(yaml_file, Loader=yaml.FullLoader)
+            self.__yaml = yaml.load(yaml_file, Loader=yaml.FullLoader)
 
         # Check for the output path
-        if (self.yaml["output_path"] is not None) and \
-                (not os.path.exists(self.yaml["output_path"])):
-            os.makedirs(self.yaml["output_path"])
-        elif (self.yaml["output_path"] is not None) and \
-                not os.path.isdir(self.yaml["output_path"]):
+        if (self.__yaml["output_path"] is not None) and \
+                (not os.path.exists(self.__yaml["output_path"])):
+            os.makedirs(self.__yaml["output_path"])
+        elif (self.__yaml["output_path"] is not None) and \
+                not os.path.isdir(self.__yaml["output_path"]):
             LOGGER.error("Provided output path is already exist and not a directory. "
                          "Please check for the input file.")
             sys.exit(1)
 
         # Image to show, init to None
-        self.frame = None
+        self.__frame = None
 
         # Initialize the root window and image panel
         self.root = tk.Tk()
@@ -267,51 +334,53 @@ class Interface:
         self.root.minsize(850, 500)
 
         # Open the detection Path
-        self.__detect = Detection(google_kit_json_path=self.yaml["google-kit-json"],
-                                  categories=self.yaml["categories"],
-                                  size=(self.yaml["width"], self.yaml["height"]),
-                                  max_results=int(self.yaml["max_request"]),
-                                  camera=self.yaml["camera"],
+        self.__detect = Detection(google_kit_json_path=self.__yaml["google-kit-json"],
+                                  categories=self.__yaml["categories"],
+                                  size=(self.__yaml["width"], self.__yaml["height"]),
+                                  max_results=int(self.__yaml["max_request"]),
+                                  camera=self.__yaml["camera"],
                                   )
-        self.__detect.start()
 
         # Check OS to make the design
         if platform.system() == "Darwin":
             # Design the button of "snapshot" for MACOSX
-            snap_btn = tkmacosx.Button(self.root,
-                                       text="Snapshot!",
-                                       command=self.take_snapshot,
-                                       padx=10,
-                                       pady=10,
-                                       bg="#EDD4B2")
+            self.__snap_btn = tkmacosx.Button(self.root,
+                                              text="Snapshot!",
+                                              command=self.__take_snapshot,
+                                              padx=10,
+                                              pady=10,
+                                              bg="#A8E0FF")
             # Design the button of "quit" for MACOSX
-            quit_btn = tkmacosx.Button(self.root,
-                                       text='Quit',
-                                       command=self.on_close,
-                                       padx=10,
-                                       pady=10,
-                                       background="#D0A98F")
+            self.__quit_btn = tkmacosx.Button(self.root,
+                                              text='Quit',
+                                              command=self.__on_close,
+                                              padx=10,
+                                              pady=10,
+                                              background="#778EBB")
         else:
             # Design the button of "snapshot" for other OS
-            snap_btn = tk.Button(self.root,
-                                 text="Snapshot!",
-                                 command=self.take_snapshot,
-                                 padx=10,
-                                 pady=10,
-                                 bg="#EDD4B2")
+            self.__snap_btn = tk.Button(self.root,
+                                        text="Snapshot!",
+                                        command=self.__take_snapshot,
+                                        padx=10,
+                                        pady=10,
+                                        bg="#A8E0FF")
             # Design the button of "quit" for other OS
-            quit_btn = tk.Button(self.root,
-                                 text='Quit',
-                                 command=self.on_close,
-                                 padx=10,
-                                 pady=10,
-                                 background="#D0A98F")
+            self.__quit_btn = tk.Button(self.root,
+                                        text='Quit',
+                                        command=self.__on_close,
+                                        padx=10,
+                                        pady=10,
+                                        background="#778EBB")
 
-        snap_btn.grid(row=0,
-                      column=1)
+        self.__snap_btn.grid(row=0,
+                             column=1)
+
+        self.__quit_btn.grid(row=3,
+                             column=1)
 
         # Design the label to display which categories show up on the screen
-        self.__info_label = tk.Label(bg="#DFBFA1",
+        self.__info_label = tk.Label(bg="#8EE3F5",
                                      height=20,
                                      width=16,
                                      anchor="nw",
@@ -320,30 +389,52 @@ class Interface:
         self.__info_label.grid(row=1,
                                column=1)
 
-        quit_btn.grid(row=2,
-                      column=1)
+        # Add user-defined label
+        self.__user_define_label = tk.Label(bg="#70CAD1",
+                                            height=10,
+                                            width=16,
+                                            anchor="nw",
+                                            padx=10,
+                                            pady=10)
 
         # Quit the event and start the video loop
         self.__quit = threading.Event()
-        video_start = threading.Thread(target=self.video_loop,
+        video_start = threading.Thread(target=self.__video_loop,
                                        daemon=True)
         video_start.start()
 
         # Set a callback to handle when the window is closed
         self.root.wm_title("Object detection")
-        self.root.wm_protocol("WM_DELETE_WINDOW", self.on_close)
+        self.root.wm_protocol("WM_DELETE_WINDOW", self.__on_close)
 
-    def video_loop(self):
+    @property
+    def frame(self):
+        """numpy.array: Getter of detected frame."""
+        return self.__frame
+
+    @property
+    def size(self):
+        """tuple (width, height): Size of the image and frame."""
+        return self.__yaml["size"][0], self.__yaml["size"][1]
+
+    def __video_loop(self):
+        """Show the scene that the camera captured by using open CV.
+
+        Args:
+            self: Instance itself.
+
+        Return: None
+        """
         # Keep looping over frames until we are instructed to stop
         while not self.__quit.is_set():
             # Get the correct frame
-            self.frame = self.__detect.frame
-            if self.frame is None:
+            self.__frame = self.__detect.frame
+            if self.__frame is None:
                 continue
 
             # Convert BGR color to RGB and
             #  make it to pillow format
-            image = cv2.cvtColor(self.frame, cv2.COLOR_BGR2RGB)
+            image = cv2.cvtColor(self.__frame, cv2.COLOR_BGR2RGB)
             image = Image.fromarray(image)
             image = ImageTk.PhotoImage(image)
 
@@ -353,7 +444,7 @@ class Interface:
                 self.__panel.image = image
                 self.__panel.grid(row=0,
                                   column=0,
-                                  rowspan=3,
+                                  rowspan=4,
                                   padx=10,
                                   pady=10)
             else:  # Otherwise, simply update the panel
@@ -366,34 +457,45 @@ class Interface:
                 text = f"{name}: {count}\n"
             self.__info_label.configure(text=text)
 
-    def upload(self, filename):
+    def __upload(self, filename):
+        """Save image to both Cloud and local.
+
+        Args:
+            self: Instance itself.
+            filename: The name of saved file.
+
+        Return: None
+        """
         # Save to local path
-        save_img = self.frame.copy()
+        save_img = self.__frame.copy()
+
+        # Initialize the bucket for after usage
+        image_blob = None
 
         # Make the Google Cloud Storage client
         #  and set the storage path
-        if self.yaml["bucket"] is not None:
+        if self.__yaml["bucket"] is not None:
             client = storage.Client()
-            bucket = client.get_bucket(self.yaml["bucket"])
+            bucket = client.get_bucket(self.__yaml["bucket"])
             image_blob = bucket.blob(filename)
 
         # Upload and save the image
         try:
-            if self.yaml["output_path"] is not None:
+            if self.__yaml["output_path"] is not None:
                 # Save image in local
-                LOGGER.info(f"Saved {filename} in local folder",)
-                path = os.path.sep.join((self.yaml["output_path"], filename))
+                LOGGER.info(f"Saved {filename} in local folder", )
+                path = os.path.sep.join((self.__yaml["output_path"], filename))
                 cv2.imwrite(path, save_img)
 
                 # Upload to Google Cloud Storage
                 #  if the user set the "bucket" option
-                if self.yaml["bucket"] is not None:
-                    image_blob.upload_from_filename(os.path.sep.join((self.yaml["output_path"],
+                if self.__yaml["bucket"] is not None:
+                    image_blob.upload_from_filename(os.path.sep.join((self.__yaml["output_path"],
                                                                       filename)),
                                                     content_type="image/jpeg")
 
                     LOGGER.info(f"Saved {filename} to google cloud storage")
-            elif self.yaml["bucket"] is not None:
+            elif self.__yaml["bucket"] is not None:
                 # Convert numpy array to bytes
                 temp_file = Image.fromarray(cv2.cvtColor(save_img, cv2.COLOR_BGR2RGB))
                 temp_file_bytes = io.BytesIO()
@@ -411,22 +513,56 @@ class Interface:
             #  and don't exit the program
             LOGGER.warning(error)
 
-    def take_snapshot(self):
+    def __take_snapshot(self):
+        """Take the current snapshot and call the upload function.
+
+        Args:
+            self: Instance itself.
+
+        Return: None
+        """
         # Set the filename to
         #  year_month_date_hour_minute_second.jpg
         time_label = datetime.datetime.now()
         filename = f"{time_label.strftime('%Y-%m-%d_%H-%M-%S')}.jpg"
 
         # Start the saving function in background
-        save_img_back = threading.Thread(target=self.upload,
+        save_img_back = threading.Thread(target=self.__upload,
                                          args=(filename,),
                                          daemon=True)
         save_img_back.start()
 
+    def extra_info(self, info: dict):
+        # Set up the label config
+        self.__info_label.configure(height=10)
+        self.__user_define_label.grid(row=2,
+                                      column=1)
+
+        # Put on the information
+        text = ""
+        for name, content in info.items():
+            text = f"{name}: {content}\n"
+        self.__user_define_label.configure(text=text)
+
     def start(self):
+        """Start the GUI.
+
+        Args:
+            self: Instance itself.
+
+        Return: None
+        """
+        self.__detect.start()
         self.root.mainloop()
 
-    def on_close(self):
+    def __on_close(self):
+        """End the GUI and release the resource.
+
+        Args:
+            self: Instance itself.
+
+        Return: None
+        """
         # Release the resource and
         #  close the windows
         LOGGER.info("closing...")
